@@ -276,6 +276,45 @@ where
         });
     }
 
+    /// Mark values as deleted where `callback` returns `false`
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use infinitree::fields::VersionedMap;
+    ///
+    /// let m = VersionedMap::<usize, String>::default();
+    ///
+    /// m.insert(1, "first".to_owned());
+    ///
+    /// m.retain(|k, v| false);
+    /// assert_eq!(m.contains(&1), false);
+    /// ```
+    #[inline(always)]
+    pub fn retain(&self, mut callback: impl FnMut(&K, &V) -> bool) {
+        // note: this is copy-pasta, because the closures have
+        // different lifetimes.
+        //
+        // if you have a good idea how to avoid
+        // using a macro and just do this, please send a PR
+
+        self.base.for_each(|k, v: &mut Action<V>| {
+            if let Some(value) = v {
+                if !(callback)(k, Arc::as_ref(value)) {
+                    self.current
+                        .upsert(k.clone(), || Action::None, |_, v| *v = Action::None)
+                }
+            }
+        });
+        self.current.for_each(|k, v: &mut Action<V>| {
+            if let Some(value) = v {
+                if !(callback)(k, Arc::as_ref(value)) {
+                    *v = Action::None
+                }
+            }
+        });
+    }
+
     /// Clear out the current changeset, and commit all changes to history.
     ///
     /// This operation potentially helps free some memory, but more
@@ -459,7 +498,7 @@ where
 
     #[inline(always)]
     fn insert(&mut self, record: Self::Item) {
-        debug_assert!(self.field.base.insert(record.0, record.1).is_ok());
+        assert!(self.field.base.insert(record.0, record.1).is_ok());
     }
 }
 
@@ -590,8 +629,38 @@ mod test {
     use crate::{
         fields::{LocalField, SparseField, Strategy},
         index::test::store_then_load,
+        Infinitree, Key,
     };
 
+    #[test]
+    fn bare_index_can_be_restored() {
+        let key = || Key::from_credentials("bare_index_map", "password").unwrap();
+        let storage = crate::backends::test::InMemoryBackend::shared();
+
+        {
+            let mut tree = Infinitree::<VersionedMap<usize, usize>>::empty(storage.clone(), key());
+            tree.index().insert(1000, 1000);
+            tree.commit(None);
+            tree.index().clear();
+
+            for i in 0..100 {
+                tree.index().insert(i, i + 1);
+            }
+
+            tree.commit(None);
+        }
+
+        let mut tree =
+            Infinitree::<VersionedMap<usize, usize>>::open(storage.clone(), key()).unwrap();
+        tree.load_all();
+
+        for i in 0..100 {
+            assert_eq!(i + 1, *tree.index().get(&i).unwrap());
+        }
+
+        assert_eq!(tree.index().len(), 101);
+    }
+
     #[test]
     fn duplicate_insert_is_noop() {
         let m = VersionedMap::<usize, String>::default();
