@@ -16,6 +16,9 @@ pub use reader::{AEADReader, Reader};
 mod writer;
 pub use writer::{AEADWriter, Writer};
 
+mod bufferedstream;
+pub use bufferedstream::{BufferedSink, BufferedStream, StreamChunks};
+
 pub mod serializer;
 
 mod id;
@@ -47,10 +50,18 @@ pub enum ObjectError {
     },
     #[error("Chunk too large to be written: {size}, max: {max_size}")]
     ChunkTooLarge { max_size: usize, size: usize },
+    #[error("Buffer is smaller than required size: {min_size}")]
+    BufferTooSmall { min_size: usize },
+    #[error("Fatal internal error")]
+    Fatal,
     #[error("Serialize failed")]
-    Serialize { source: Box<dyn std::error::Error> },
+    Serialize {
+        source: Box<dyn std::error::Error + Send + Sync>,
+    },
     #[error("Deserialize failed")]
-    Deserialize { source: Box<dyn std::error::Error> },
+    Deserialize {
+        source: Box<dyn std::error::Error + Send + Sync>,
+    },
 }
 
 pub type Result<T> = std::result::Result<T, ObjectError>;
@@ -376,7 +387,7 @@ impl<T> AsRef<Object<T>> for Object<T> {
     }
 }
 
-#[cfg(test)]
+#[cfg(any(test, feature = "_test"))]
 pub mod test {
     use crate::{ChunkPointer, Digest};
 
